const {
  AutojoinRoomsMixin,
  MatrixAuth,
  MatrixClient,
  RustSdkCryptoStorageProvider,
  SimpleFsStorageProvider,
} = require('matrix-bot-sdk');
const fastq = require('fastq');

const {
  MATRIX_HOMESERVER_URL,
  MATRIX_USER,
  MATRIX_PASSWORD,
  BOT_STORAGE_PATH,
  BOT_CRYPTO_STORAGE_PATH,
} = process.env;

const storageProvider = new SimpleFsStorageProvider(BOT_STORAGE_PATH);
const cryptoProvider = new RustSdkCryptoStorageProvider(BOT_CRYPTO_STORAGE_PATH);
let client;
let joinedRooms;
const processOutboundQueue = async ({ text }) => {
  joinedRooms.forEach(async (roomId) => {
    await client.sendText(roomId, text);
  });
};
const outboundQueue = fastq(processOutboundQueue, 1);
outboundQueue.pause();

const start = async () => {
  console.log('MATRIX_HOMESERVER_URL:', MATRIX_HOMESERVER_URL);
  const auth = new MatrixAuth(MATRIX_HOMESERVER_URL);
  const authClient = await auth.passwordLogin(MATRIX_USER, MATRIX_PASSWORD);
  client = new MatrixClient(
    MATRIX_HOMESERVER_URL,
    authClient.accessToken,
    storageProvider,
    cryptoProvider,
  );

  // Automatically join a room to which we are invited
  AutojoinRoomsMixin.setupOnClient(client);

  joinedRooms = await client.getJoinedRooms();
  console.log('joined rooms:', joinedRooms);

  async function handleCommand(roomId, event) {
    // Don't handle unhelpful events (ones that aren't text messages, are redacted, or sent by us)
    if (event.content?.msgtype !== 'm.text') return;
    if (event.sender === await client.getUserId()) return;

    // Check to ensure that the `!hello` command is being run
    const { body } = event.content;
    if (!body?.startsWith('!hello')) return;

    // Now that we've passed all the checks, we can actually act upon the command
    await client.replyNotice(roomId, event, 'Hello world!');
  }

  // Before we start the bot, register our command handler
  client.on('room.message', handleCommand);

  client.start().then(() => {
    console.log('Bot started!');
    outboundQueue.resume();
  });
};

const broadcastMessage = (text) => {
  outboundQueue.push({ text });
};

module.exports = {
  start,
  broadcastMessage,
};
